% % % % % This document provides an example demo for the JOULE model formulated in
% % % % % our TPAMI paper. Please cite the following paper if you use the data and codes in your research
% % % % 
% % % % % [1] Jian-Fang Hu, Wei-Shi Zheng, Jianhuang Lai, and Jianguo Zhang, Jointly Learning Heterogeneous Features
% % % % %     for RGB-D Activity Recognition, IEEE Trans. on Pattern Analysis and Machine Intelligence, (Accepted)
% % % % % [2] Jian-Fang Hu, Wei-Shi Zheng, Jianhuang Lai, and Jianguo Zhang, Jointly Learning Heterogeneous Features
% % % % %     for RGB-D Activity Recognition,  IEEE Conference on Computer Vision and Pattern Recognition (CVPR),
% % % % %     5344-5352 (2015).
% % % % % Updated by Jian-Fang Hu in December 5, 2016.

%% set information
sampleNum=480;
selTrainVideo=1:240;  selTestVideo=241:sampleNum;
videoClassT=repmat(1:12,20,1); videoClass=[videoClassT(:);videoClassT(:)];

%% generate and preprocess heterogeneous features
featureNum=6;
XTmp={};
for ii=1:featureNum
   XTmp{ii}=randn(1000+ii*100,sampleNum); 
end

%% preprocess
Ytmp=-ones(length(unique(videoClass)),size(XTmp{1},2));
for ii=1:length(videoClass)
    Ytmp(videoClass(ii),ii)=length(unique(videoClass))-1;
end

XTrain={};YTrain={};XTest={};
for ii=1:length(XTmp)
   YTrain{ii}=Ytmp(:,selTrainVideo);                   % Label matrices
   dataTmp=XTmp{ii}(:,selTrainVideo);
   dataTmpTest=XTmp{ii}(:,selTestVideo);
   [datanew, dataTmpTest]=submean(dataTmp,dataTmpTest);
   XTrain{ii}=datanew;                                 % Features for training
   XTest{ii}=dataTmpTest;                              % Features for testing
end

%% optimization
M=50; alpha=0.1; gamma=0.1; lambda=0.5; maxIterOut=20; maxIterIn=50;
[W,Theta,W0,objvals,specificTrain,specificTest,shareTrain,shareTest]=...
    mtlProjectVOrthogonalityHingLossExtThreeSteps(XTrain,XTest,YTrain...
    ,M,alpha,gamma,lambda,maxIterOut,maxIterIn);
